#include<Inventor/nodes/SoCoordinate3.h>
#include<Inventor/nodes/SoSeparator.h>
#include<Inventor/nodes/SoNormal.h>
#include<Inventor/nodes/SoNormalBinding.h>
#include<Inventor/nodes/SoMaterial.h>
#include<Inventor/nodes/SoMaterialBinding.h>
#include<Inventor/nodes/SoIndexedFaceSet.h>
#include<Inventor/nodes/SoFaceSet.h>
#include<Inventor/nodes/SoTransform.h>
#include<Inventor/nodes/SoScale.h>

// Positions of all of the vertices:
//
static float vertexPositions[12][3] =
{
{ 0.0000f, 1.2142f, 0.7453f}, // top
{ 0.0000f, 1.2142f, -0.7453f}, // points surrounding top
{-1.2142f, 0.7453f, 0.0000f},
{-0.7453f, 0.0000f, 1.2142f},
{ 0.7453f, 0.0000f, 1.2142f},
{ 1.2142f, 0.7453f, 0.0000f},
{ 0.0000f, -1.2142f, 0.7453f}, // points surrounding bottom
{-1.2142f, -0.7453f, 0.0000f},
{-0.7453f, 0.0000f, -1.2142f},
{ 0.7453f, 0.0000f, -1.2142f},
{ 1.2142f, -0.7453f, 0.0000f},
{ 0.0000f, -1.2142f, -0.7453f}, // bottom
};

// Connectivity, information; 12 faces with 5 vertices each },
// (plus the end-of-face indicator for each face):
static int indices[72] =
{
1, 2, 3, 4, 5, SO_END_FACE_INDEX, // top face
0, 1, 8, 7, 3, SO_END_FACE_INDEX, // 5 faces about top
0, 2, 7, 6, 4, SO_END_FACE_INDEX,
0, 3, 6, 10, 5, SO_END_FACE_INDEX,
0, 4, 10, 9, 1, SO_END_FACE_INDEX,
0, 5, 9, 8, 2, SO_END_FACE_INDEX,
9, 5, 4, 6, 11, SO_END_FACE_INDEX, // 5 faces about bottom
10, 4, 3, 7, 11, SO_END_FACE_INDEX,
6, 3, 2, 8, 11, SO_END_FACE_INDEX,
7, 2, 1, 9, 11, SO_END_FACE_INDEX,
8, 1, 5,10, 11, SO_END_FACE_INDEX,
6, 7, 8, 9, 10, SO_END_FACE_INDEX, // bottom face
};
// Colors for the 12 faces
static float colors[12][3] =
{
{1.0f, .0, 0}, { .0, .0, 1.0f}, {0, .7f, .7f}, { .0, 1.0f, 0},
{ .7f, .7f, 0}, { .7f, .0, .7f}, {0, .0, 1.0}, { .7f, .0, .7f},
{ .7f, .7f, 0}, { .0, 1.0f, .0}, {0, .7f, .7f}, {1.0f, .0, 0}
};
// Routine to create a scene graph representing a dodecahedron
SoSeparator *
makeStellatedDodecahedron(float x, float y, float z)
{
SoSeparator *Dodecahedron = new SoSeparator;
Dodecahedron->ref();

  SoTransform *transform = new SoTransform;
  transform->translation.setValue(x,y,z);
  Dodecahedron->addChild(transform);

  SoScale *scale = new SoScale;
  scale->scaleFactor.setValue(4.f,4.f,4.f);
  Dodecahedron->addChild(scale);

// Define colors for the faces
SoMaterial *myMaterials = new SoMaterial;
myMaterials->diffuseColor.setValues(0, 12, colors);
Dodecahedron->addChild(myMaterials);
SoMaterialBinding *myMaterialBinding = new SoMaterialBinding;
myMaterialBinding->value = SoMaterialBinding::PER_FACE;
Dodecahedron->addChild(myMaterialBinding);
// Define coordinates for vertices
// Define coordinates for vertices
SoCoordinate3 *myCoords = new SoCoordinate3;
myCoords->point.setValues(0, 12, vertexPositions);
Dodecahedron->addChild(myCoords);
// Define the IndexedFaceSet, with indices into
// the vertices:
SoIndexedFaceSet *myFaceSet = new SoIndexedFaceSet;
myFaceSet->coordIndex.setValues(0, 72, indices);
Dodecahedron->addChild(myFaceSet);
Dodecahedron->unrefNoDelete();
return Dodecahedron;
}