//
// Pklad 3.6 - Genertor krajiny fault formation algoritmem s mnohmi vylepenmi
//


#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoDirectionalLight.h>
#include <Inventor/nodes/SoEnvironment.h>
#include <Inventor/nodes/SoIndexedTriangleStripSet.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoMaterialBinding.h>
#include <Inventor/nodes/SoNormal.h>
#include <Inventor/nodes/SoNormalBinding.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoTriangleStripSet.h>
#include <stdlib.h>
#include <time.h>
#include "FaultFormation2.h"
#include "GenTown.h"

#ifdef _MSC_VER
float strtof(const char *nptr, char **endptr)  { return float(strtod(nptr, endptr)); }
#endif


// Globln promnn a funkce,
// kter jsou pouity v modulech Main-*.cpp
const char *appTitle = "Virtualni mestska zastavba v krajine Fault Formation 2"; // Titulek okna
bool headlight = true; //false;                           // Defaultn svtlo (v pozici kamery)
SoSeparator* createGener(int argc, char **argv);  // Funkce vytvejc scnu

// Velikost vkov mapy zvten o jednu
// (krajina 64x64 se konstruuje ze 65x65 bod)
#define DEFAULT_TERRAIN_HMAP_WIDTH  129
#define DEFAULT_TERRAIN_HMAP_HEIGHT 129

// Velikost generovan krajiny v prostoru
#define DEFAULT_TERRAIN_WSIZE 128.f
#define DEFAULT_TERRAIN_HSIZE 128.f

// Parametry genertoru vkov mapy
#define DEFAULT_TERRAIN_NUM_ITER  150
#define DEFAULT_TERRAIN_MAX_DELTA 2.f
#define DEFAULT_TERRAIN_MIN_DELTA 0.05f

// Filtrovac koeficient pro finln vyhlazen krajiny.
// Vy slo odpovd vy "hrbolatosti",
// ni - vce se to vyfiltruje.
#define DEFAULT_TERRAIN_FILTER_Q  0.85f

// HQ (high quality) uruje, jakm zpsobem bude rendrovna vsledn krajina.
// HQ=0 znamen, e kad tverec bude rendrovn jako dva trojhelnky,
// kdeto HQ=1 pid do stedu tverce dal vertex a vyrendruje
// tverec tymi trojhelnky. Pesto, e to m za dsledek dvojnsobn
// mnostv trojhelnk, krajina se stane hlad.
#define DEFAULT_TERRAIN_HQ  0

TerrainPatch terrain;

/*!
  Funkce vrac index do vkov mapy hmap pro souadnice [x,y].
*/
inline int TerrainPatch::getHmapIndex(const int x, const int y)
{
  return y*HMapWidth + x;
}



//-----------------------------------------------------------------------------
/*!
  Generuje vyskovou mapu pomoci Fault-Formation algoritmu.

  Parametry: MaxDelta - maximalni mozna velikost zlomu
             MinDelta - minimalni velikost zlomu
             NumIters - pocet zlomu
             FilterValue - hodnota vyhlazovani: <0f-1f> cim vyssi, tim mensi

  Algoritmus: Dva nahodne vybrane body urci vektor "zlomu" terenu. Cela plocha
              na jedne strane zlomu je nasledne zvysena o hodnotu Delta.
              Delta se s rostoucim poctem iteraci snizuje. Po vygenerovani
              je na mapu aplikovan filtr, ktery teren "uhladi".
*/
void TerrainPatch::generateHeightMap(int hmapWidth, int hmapHeight, int NumIter,
                                     float MaxDelta, float MinDelta, float FilterValue,
									 float wsize, float hsize)
{
  // 

  this -> wsize = wsize;
  this -> hsize = hsize;


  // Alokujeme vkovou mapu
  allocHMap(hmapWidth, hmapHeight);

  // init mapy, algoritmus predpoklada pocatecni vynulovani
  for (unsigned int i = 0; i < HMapWidth*HMapHeight; i++)
    HMap[i] = 0.f;

  // Fault Formation algoritmus
  float Delta = 0;

  unsigned int Rand1X, Rand1Y, Rand2X, Rand2Y;
  int FaultVectorX, FaultVectorY;
  int ToPointVectorX, ToPointVectorY;

  for (int iter = 0; iter < NumIter; iter++) {
    Rand1X = rand() % HMapWidth;
    Rand1Y = rand() % HMapHeight;
    Rand2X = rand() % HMapWidth;
    Rand2Y = rand() % HMapHeight;

    // vektor zlomu
    FaultVectorX = Rand2X - Rand1X;
    FaultVectorY = Rand2Y - Rand1Y;

    // hodnota, ktera bude pridana na jednu stranu od zlomu
    Delta = (float(MaxDelta - ((MaxDelta-MinDelta) * iter))) / NumIter;

    for (unsigned int x = 0; x < HMapWidth; x++)
      for (unsigned int y = 0; y < HMapHeight; y++) {
        // vektor k aktualni pozici na mape
        ToPointVectorX = x - Rand1X;
        ToPointVectorY = y - Rand1Y;
        // z-ova slozka vektoroveho soucinu urci, na ktere strane od
        // "zlomu" se aktualni bod nachazi
        if ((ToPointVectorX*FaultVectorY-FaultVectorX*ToPointVectorY) > 0)
          HMap[getHmapIndex(x,y)] += Delta;
      }
  }
  // aplikuje se filtr
  for (int j=0; j<10; j++)
    filter(FilterValue);
};



//-----------------------------------------------------------------------------
/*!
  Generuje vyskovou mapu pomoci Fault-Formation algoritmu,
  piem pouv defaultn nastaven parametr.
 */
void TerrainPatch::generateHeightMap()
{
  generateHeightMap(DEFAULT_TERRAIN_HMAP_WIDTH, DEFAULT_TERRAIN_HMAP_HEIGHT,
	                DEFAULT_TERRAIN_NUM_ITER, DEFAULT_TERRAIN_MAX_DELTA,
                    DEFAULT_TERRAIN_MIN_DELTA, DEFAULT_TERRAIN_FILTER_Q, 
					DEFAULT_TERRAIN_WSIZE, DEFAULT_TERRAIN_HSIZE);
};




//  my code ----------------------------------------
void TerrainPatch::getLeveL(float wsize, float hsize, int hq)
{
  // Vpoet minHeight a maxHeight
  minHeight = maxHeight = HMap[0];
  unsigned int i;
  for (i=1; i<HMapWidth*HMapHeight; i++) {
    if (HMap[i] < minHeight)
      minHeight = HMap[i];
    if (HMap[i] > maxHeight)
      maxHeight = HMap[i];
  }

  level.water=(maxHeight - minHeight) / 5.f + minHeight;
  level.mountains=minHeight+3.f*(maxHeight - minHeight) / 5.f; 
  level.snow= minHeight+4.f*(maxHeight - minHeight) / 5.f;
  printf("waterLevel: %f\n",level.water);
  printf("mountains: %f\n",level.mountains);
  printf("snow: %f\n",level.snow);

}
//-----------------------------------------------------------------------------



/*!
  Ziska vykreslitelny model povrchu
*/
SoSeparator* TerrainPatch::getModel(float wsize, float hsize, int hq)
{
  if (!HMap)
    generateHeightMap();

  // Vpoet minHeight a maxHeight
  minHeight = maxHeight = HMap[0];
  unsigned int i;
  for (i=1; i<HMapWidth*HMapHeight; i++) {
    if (HMap[i] < minHeight)
      minHeight = HMap[i];
    if (HMap[i] > maxHeight)
      maxHeight = HMap[i];
  }

  SoSeparator *root = new SoSeparator;

  // Vytvo model vodn hladiny
  // Nejprve materil, pak souadnice, a na zvr stripSet
  SoMaterial *waterMat = new SoMaterial;
  waterMat->ambientColor.setValue(0.f, 0.f, 1.f);
  waterMat->diffuseColor.setValue(0.f, 0.f, 0.f);
  root->addChild(waterMat);

  SoCoordinate3 *waterCoords = new SoCoordinate3;
  float waterLevel = (maxHeight - minHeight) / 5.f + minHeight; // voda



  // my code ---------------------
  level.water=waterLevel;
  level.mountains=minHeight+3.f*(maxHeight - minHeight) / 5.f; 
  level.snow= minHeight+4.f*(maxHeight - minHeight) / 5.f;
  printf("waterLevel: %f\n",level.water);
  printf("mountains: %f\n",level.mountains);
  printf("snow: %f\n",level.snow);
  //printf("%f",level.water+(level.mountains - level.water)/2.f);
  // -----------------------------



  waterCoords->point.set1Value(0, SbVec3f(0.f,   waterLevel, 0.f));
  waterCoords->point.set1Value(1, SbVec3f(wsize, waterLevel, 0.f));
  waterCoords->point.set1Value(2, SbVec3f(0.f,   waterLevel, hsize));
  waterCoords->point.set1Value(3, SbVec3f(wsize, waterLevel, hsize));
  root->addChild(waterCoords);

  SoTriangleStripSet *waterStripSet = new SoTriangleStripSet;
  waterStripSet->numVertices.set1Value(0, 4); // nult prvek nastav na hodnotu 4,
      // to znamen, e budeme renderovat 2 trojhelnky (poet bod snme o
      // dva a zskme tak poet trojhelnk
  root->addChild(waterStripSet);

  // definice materialu (barvy) krajiny
  SoMaterial *material = new SoMaterial;
  // trava "pod vodou"
  material->ambientColor.set1Value(0, SbColor(0.9f, 0.9f, 0.9f));
  material->diffuseColor.set1Value(0, SbColor(0.0f, 0.6f, 0.0f));
  // trava
  material->ambientColor.set1Value(1, SbColor(0.9f, 0.9f, 0.9f));
  material->diffuseColor.set1Value(1, SbColor(0.0f, 0.6f, 0.0f));
  // skala
  material->ambientColor.set1Value(2, SbColor(0.9f, 0.9f, 0.9f));
  material->diffuseColor.set1Value(2, SbColor(0.6f, 0.3f, 0.0f));
  // snih
  material->ambientColor.set1Value(3, SbColor(0.9f, 0.9f, 0.9f));
  material->diffuseColor.set1Value(3, SbColor(0.6f, 0.6f, 0.6f));
  root->addChild(material);

  // vytvor nody pro souradnice a normaly
  SoCoordinate3 *coords = new SoCoordinate3;
  SoNormal *normals = new SoNormal;

  // nastavime pocet souradnic a normal, ktere budeme generovat
  int numVertices = (hq!=2) ? HMapWidth*HMapHeight : (HMapWidth*HMapHeight)+((HMapWidth-1)*(HMapHeight-1));
  coords->point.setNum( numVertices );
  normals->vector.setNum( numVertices );

  // binding pro normaly - v nasem pripade PER_VERTEX_INDEXED
  SoNormalBinding *nbind = new SoNormalBinding;
  nbind->value.setValue(SoNormalBinding::PER_VERTEX_INDEXED);
  root->addChild(nbind);

  // binding pro materily
  SoMaterialBinding *mbind = new SoMaterialBinding;
  mbind->value.setValue(SoMaterialBinding::PER_VERTEX_INDEXED);
  root->addChild(mbind);

  // ukazatele na souradnice a normaly
  SbVec3f *c = coords->point.startEditing();
  SbVec3f *n = normals->vector.startEditing();

  unsigned int x, y;
  float sx = wsize / (HMapWidth-1);
  float sy = hsize / (HMapHeight-1);

  switch (hq) {
  case 0:
  case 1: {

    // GENEROVANI SOURADNIC A NORMAL
    // Low-Quality BruteForce
    // Kazdy ctverec povrchu je tvoren dvema trojuhelniky

    // vertexy odpovidajici jednotlivym pixelum na mape
    for (unsigned int y = 0; y < HMapHeight; y++) {
      for (unsigned int x = 0; x < HMapWidth; x++) {
        *(c++) = SbVec3f(float(x)*sx,float(HMap[getHmapIndex(x,y)]),float(y)*sy);
        *(n++) = calculateNormal(x,y, sx,sy);
      }
    }
    break;
  }
  case 2: {

    // GENEROVANI SOURADNIC A NORMAL
    // High-Quality BruteForce
    // Kazdy ctverec povrchu je tvoren ctyrmi trojuhelniky, je nutne zahrnout
    // mezi vertexy i "stred" ctverce, kazdy ctverec je kvuli tomu tvoren
    // dvema triangle_stripy

    // vertexy odpovidajici jednotlivym pixelum na mape
    for (y = 0; y < HMapHeight; y++) {
      for (x = 0; x < HMapWidth; x++) {
        *(c++) = SbVec3f(float(x)*sx,float(HMap[getHmapIndex(x,y)]),float(y)*sy);
        *(n++) = calculateNormal(x,y, sx,sy);
      }
    }

    // vertexy odpovidajici bodum "uprostred" jednotlivych ctvercu
    float value = 0;
    SbVec3f normala;
    for (y = 0; y < HMapHeight-1; y++) {
      for (x = 0; x < HMapWidth-1; x++) {
        value = ( HMap[getHmapIndex(x,y)] + HMap[getHmapIndex(x+1,y)] +
                  HMap[getHmapIndex(x,y+1)] + HMap[getHmapIndex(x+1,y+1)] ) / 4;
        *(c++) = SbVec3f(float(x+0.5), float(value), float(y+0.5));

        normala = calculateNormal(x,y, sx,sy) + calculateNormal(x,y+1, sx,sy)
                  + calculateNormal(x+1,y+1, sx,sy) + calculateNormal(x+1,y, sx,sy);

        normala.normalize();
        *(n++) = normala;
      }
    }
    break;
  }
  }

  // zaradime vytvorene uzly do grafu sceny
  coords->point.finishEditing();
  normals->vector.finishEditing();
  root->addChild(normals);
  root->addChild(coords);

  // konfigurace SoIndexedTriangleStripSetu
  SoIndexedTriangleStripSet *stripSet = new SoIndexedTriangleStripSet;

  // nastavime pocet indexu, ktere budeme generovat
  int numIndices;
  switch (hq) {
  case 0: numIndices = (2*HMapWidth+1)*(HMapHeight-1); break;
  case 1: numIndices = 5*(HMapWidth-1)*(HMapHeight-1); break;
  case 2: numIndices = 10*(HMapWidth-1)*(HMapHeight-1); break;
  }
  stripSet->coordIndex.setNum( numIndices );
  stripSet->materialIndex.setNum( numIndices );

  // ukazatele na indexy (normalIndexy jsou shodne s coordIndexy,
  // nemusi se nastavovat)
  int32_t *ci = stripSet->coordIndex.startEditing();
  int32_t *mi = stripSet->materialIndex.startEditing();

  int32_t index = 0;
  long distance = 0;

  switch (hq) {
  case 0: {

    // GENEROVANI INDEXU SOURADNIC
    // Low-Quality
    // kazda dlazdice terenu je tvorena dvema trojuhelniky,
    // pricemz cely jeden radek krajiny je rendrovan jako jeden triangle strip

    index = 0;
    for (unsigned int strip = 0; strip < (HMapHeight-1); strip++) {
      for (unsigned int column = 0; column < HMapWidth; column++) {
        ci[index] = (strip*HMapWidth)+column;
        mi[index] = getMaterialIndex(HMap[strip*HMapWidth+column], minHeight, maxHeight);
        index++;
        ci[index] = ci[index-1] + HMapWidth;
        mi[index] = getMaterialIndex(HMap[(strip+1)*HMapWidth+column], minHeight, maxHeight);
        index++;
      }
      ci[index] = -1; // konec Triangle_stripu
      mi[index] = -1;
      index++;
    }
    break;
  }
  case 1: {

    // GENEROVANI INDEXU SOURADNIC
    // Low-Quality, adaptive tesselation
    // kazda dlazdice terenu je tvorena dvema trojuhelniky
    // rendrovanymi jako jeden triangle strip,
    // zlom mezi trojuhelniky je generovan podle smeru vetsi zmeny vysky v dlazdici

    index = 0;
    int i1 = 0;
    int i2 = HMapWidth;
    for (unsigned int strip = 0; strip < HMapHeight-1; strip++) {
      for (unsigned int column = 0; column < HMapWidth-1; column++) {
        // potej zmnu v jednotlivch smrech dladice,
        // abychom zjistili, ve kterm smru bude dladice rozdlena.
        bool orderChange = fabsf(HMap[i2+1]-HMap[i1]) < fabsf(HMap[i2]-HMap[i1+1]);
        if (orderChange) {
          int tmp = i2;
          i2 = i1;
          i1 = tmp;
        }

        ci[index] = i1;
        mi[index] = getMaterialIndex(HMap[i1++], minHeight, maxHeight);
        index++;
        ci[index] = i2;
        mi[index] = getMaterialIndex(HMap[i2++], minHeight, maxHeight);
        index++;
        ci[index] = i1;
        mi[index] = getMaterialIndex(HMap[i1], minHeight, maxHeight);
        index++;
        ci[index] = i2;
        mi[index] = getMaterialIndex(HMap[i2], minHeight, maxHeight);
        index++;
        ci[index] = -1; // konec Triangle_stripu
        mi[index] = -1;
        index++;
      }
      i1++;
      i2++;
    }
    break;
  }
  case 2: {

  // GENEROVANI INDEXU SOURADNIC
  // High-Quality

    index = 0;
    for (y = 0; y < HMapHeight-1; y++) {
      for (x = 0; x < HMapWidth-1; x++) {
        distance = (HMapWidth*HMapHeight)-y;
        int mapIndex = (x+(y*HMapWidth));
        ci[index] = mapIndex;
        mi[index] = getMaterialIndex(HMap[mapIndex], minHeight, maxHeight);
        ci[index+1] = mapIndex + HMapWidth;
        mi[index+1] = getMaterialIndex(HMap[mapIndex+HMapWidth], minHeight, maxHeight);
        ci[index+2] = mapIndex + distance;
        mi[index+2] = getMaterialIndex((HMap[getHmapIndex(x,y)] + HMap[getHmapIndex(x+1,y)] +
                                        HMap[getHmapIndex(x,y+1)] + HMap[getHmapIndex(x+1,y+1)]) / 4, minHeight, maxHeight);
        ci[index+3] = mapIndex + HMapWidth + 1;
        mi[index+3] = getMaterialIndex(HMap[mapIndex+HMapWidth+1], minHeight, maxHeight);
        ci[index+4] = -1; // konec Triangle_stripu
        mi[index+4] = -1;
        ci[index+5] = mapIndex;
        mi[index+5] = mi[index];
        ci[index+6] = ci[index+2];
        mi[index+6] = mi[index+2];
        ci[index+7] = mapIndex + 1;
        mi[index+7] = getMaterialIndex(HMap[mapIndex+1], minHeight, maxHeight);
        ci[index+8] = ci[index+3];
        mi[index+8] = mi[index+3];
        ci[index+9] = -1;  // konec Triangle_stripu
        mi[index+9] = -1;
        index+=10;
      }
    }
    break;
  }
  }

  // zaradime vytvoreny StripSet do grafu sceny
  stripSet->coordIndex.finishEditing();
  stripSet->materialIndex.finishEditing();
  root->addChild(stripSet);

  return root;
};



//-----------------------------------------------------------------------------
/*!
  Ziska vykreslitelny model povrchu
*/
SoSeparator* TerrainPatch::getModel()
{
  return getModel(DEFAULT_TERRAIN_WSIZE, DEFAULT_TERRAIN_HSIZE,
                   DEFAULT_TERRAIN_HQ);
}



//-----------------------------------------------------------------------------
/*!
  Funkce vytvo graf scny reprezentujc tverec krajiny.
  Parametry krajiny jsou pouity standardn.
*/
SoSeparator* TerrainPatch::generate()
{
  TerrainPatch terrain;
  terrain.generateHeightMap();
  return terrain.getModel();
}



//-----------------------------------------------------------------------------
/*!
  Funkce vytvo graf scny reprezentujc tverec krajiny.
*/
SoSeparator* TerrainPatch::generate(int hmapWidth, int hmapHeight, float wsize,
                               float hsize, int NumIter, float MaxDelta,
                               float MinDelta, float FilterValue, int hq)
{
  TerrainPatch terrain;
  terrain.generateHeightMap(hmapWidth, hmapHeight, NumIter, MaxDelta, MinDelta, FilterValue, wsize, hsize);
  return terrain.getModel(wsize, hsize, hq);
}



//-----------------------------------------------------------------------------
//
// Filtr kompletni plochy mapy podle zadaneho koeficientu.
// Parametr koef - <0f-1f> cim vyssi, tim mensi zmena (povrch je mene hladky)
void TerrainPatch::filter(float koef)
{
  // pokud je (koef = 1) nedochazi ke zmenam
  if (koef < 1.0f) {

    int x, y, base;

    // radky
    for (y = 0; y < static_cast<int>(HMapHeight); y++) {
      // radky zleva doprava
      base = y*HMapWidth;
      x = 1;
      while (x < static_cast<int>(HMapWidth)) {
        HMap[base+x] = static_cast<float>(HMap[base+x-1] * (1-koef) + HMap[base+x] * koef);
        x++;
      }
      // radky zprava doleva
      x = static_cast<int>(HMapWidth)-2;
      while (x > -1) { // tzn. minimalne 0 projde !!!
        HMap[base+x] = static_cast<float>(HMap[base+x+1] * (1-koef) + HMap[base+x] * koef);
        x--;
      }
    }

    // sloupce
    for(x = 0; x < static_cast<int>(HMapWidth); x++) {
      // sloupce shora dolu
      base = HMapWidth;
      while (base < static_cast<int>(HMapHeight*HMapWidth)) {
        HMap[base+x] = static_cast<float>(HMap[base+x-HMapWidth] * (1-koef) + HMap[base+x] * koef);
        base += HMapWidth;
      }
      // sloupce zespod nahoru
      base = HMapWidth * (HMapHeight-2);
      while (base > -1) { // tzn. minimalne 0 projde !!!
        HMap[base+x] = static_cast<float>(HMap[base+x+HMapWidth] * (1-koef) + HMap[base+x] * koef);
        base -= HMapWidth;
      }

    } // for

  } // if (koef < 1.0f) - pokud je (koef = 1) nedochazi ke zmenam
};



//-----------------------------------------------------------------------------
void TerrainPatch::allocHMap(unsigned int w, unsigned int h)
{
  if (HMap == NULL || HMapWidth != w || HMapHeight != h) {
    delete[] HMap;
    HMapWidth = w;
    HMapHeight = h;
    HMap = new float[HMapWidth*HMapHeight];
  }
};



//-----------------------------------------------------------------------------
void TerrainPatch::freeHMap()
{  delete[] HMap;
  HMap = NULL;
}



//-----------------------------------------------------------------------------
// Funkce vypot normlu v danm bod mapy.
SbVec3f TerrainPatch::calculateNormal(const unsigned int x, const unsigned int y,
                                      const float sx, const float sy)
{
  float dx,dy;

  // hel v ose X
  if (x>=1 && x<=HMapWidth-2)
    // Vpoet ze dvou sousednch bod
    dx = (HMap[getHmapIndex(x-1,y)] - HMap[getHmapIndex(x+1,y)]) / 2.f;
  else
    // Vpoet na okraji mapy
    if (x<1)  dx = HMap[getHmapIndex(x,y)] - HMap[getHmapIndex(x+1,y)];
    else  dx = HMap[getHmapIndex(x-1,y)] - HMap[getHmapIndex(x,y)];

  // hel v ose Y
  if (y>=1 && y<=HMapHeight-2)
    dy = (HMap[getHmapIndex(x,y-1)] - HMap[getHmapIndex(x,y+1)]) / 2.f;
  else
    if (y<1)  dy = HMap[getHmapIndex(x,y)] - HMap[getHmapIndex(x,y+1)];
    else  dy = HMap[getHmapIndex(x,y-1)] - HMap[getHmapIndex(x,y)];

  SbVec3f r(dx/sx, 1.f, dy/sy);
  r.normalize();

  return r;
};



/*!
  Funkce vrt index materilu pro dan bod vkov mapy.
 */
int TerrainPatch::getMaterialIndex(const float height, const float minHeight, const float maxHeight)
{
  float part = (maxHeight-minHeight) / 5.f;
  if (height <= minHeight+1.f*part) return 0;  // zelena pod   
  if (height <= minHeight+3.f*part) return 1;  // zelena nad
  if (height <= minHeight+4.f*part) return 2;  // hneda
  return 3; // snih
}



/*!
  Konstruktor.
*/
TerrainPatch::TerrainPatch()
{
  HMap = NULL;
}



/*!
  Destruktor.
*/
TerrainPatch::~TerrainPatch()
{
  reset();
}




/*!
  Inicializuje objekt do podobnho stavu jako pi vytvoen
  objektu, tzn. minimln vyuit zdroj.
  Funkce se pouv pedevm pro uvolnn pamti
  vkov mapy, nen-li ji dle potebn.
*/
void TerrainPatch::reset()
{
  freeHMap();
}




// Funkce pouit v ablonov funkci getParam pro konverzi
// string do rznch typ.
void strtoX(int &r, const char *nptr, char **endptr)
{
  r = strtol(nptr, endptr, 10);
}
void strtoX(float &r, const char *nptr, char **endptr)
{
  r = strtof(nptr, endptr);
}
void strtoX(bool &r, const char *nptr, char **endptr)
{
  int i;
  strtoX(i, nptr, endptr);
  r = (i != 0);
}

// ablonov funkce pro ten parametr rznch typ z pkazov dky
template<class Type>
bool getParam(int argc, char **argv, int &position,
              const char *param, Type &result)
{
  if (argc > position && strcmp(argv[position], param) == 0) {
    position++;
    if (argc > position) {
      char *errp;
      Type tmp;
      strtoX(tmp, argv[position], &errp);
      if (*(argv[position]) != '\0' && *errp == '\0') {
        result = tmp;
        position++;
        return true;
      } else {
#if !defined(__WIN32__) && !defined(_WIN32)
        printf("Chybn argument \"%s\".", param);
#else
        printf("Chybny argument \"%s\".", param);
#endif
        exit(EXIT_FAILURE);
      }
    } else {
#if !defined(__WIN32__) && !defined(_WIN32)
      printf("Chybjc hodnota pro parametr \"%s\".", param);
#else
      printf("Chybejici hodnota pro parametr \"%s\".", param);
#endif
      exit(EXIT_FAILURE);
    }
  }
  return false;
}



// Funkce vytvo graf scny pro nai aplikaci.
SoSeparator* createGener(int argc, char **argv)
{
  int i = 1;

  // Vpis npovdy
  if (argc > 1 && (strcmp(argv[1], "-h") == 0 || strcmp(argv[1], "--help") == 0)) {
#if !defined(__WIN32__) && !defined(_WIN32)
    // Standardn vpis textu na Linux/Unixech (kdovn 8852-2)
    printf("Pouit: %s [-h|--help] [-seed int] [-size float] [-heightQ float]\n" \
           "         [-dimension int] [-filterQ float] [-hq int]\n"
           "-h, --help        tato obrazovka\n"
           "-seed int         nastaven genertoru nhodnch sel\n"
           "-hmapWidth  int\n"
           "-hmapHeight int   rozmr i rozlien vkov mapy\n"
           "-wsize float\n"
           "-hsize float      ka a dlka krajiny (v prostorovch souadnicch)\n"
           "-numIter int      poet iterac fault formation algoritmu\n"
           "-maxDelta float\n"
           "-minDelta float   vka prvnho a poslednho zlomu v krajin\n"
           "-filterQ float    parametr filtrovac funkce\n"
           "-hq int           hodnota 0,1 a 2 mn reim tesselace krajiny\n"
           "                  (0 standard, 1 adaptivn smr hrany, 2 tyi trojhelnky)\n"
           "Pklad pouit (se standardnmi hodnotami parametr):\n"
           "%s -seed 10 -hmapWidth 65 -hmapHeight 65 -wsize 64 -hsize 64\n"
           "-numIter 150 -maxDelta 2 -minDelta 0.05 -filterQ 0.85 -hq 0\n",
           argv[0], argv[0]);
#else
    // Windowsovsk konzole pouv jinou znakovou sadu,
    // proto provedeme vpis bez diakritiky
    printf("Pouziti: %s [-h|--help] [-seed int] [-size float] [-heightQ float]\n" \
           "         [-dimension int] [-filterQ float] [-hq int]\n"
           "-h, --help        tato obrazovka\n"
           "-seed int         nastaveni generatoru nahodnych cisel\n"
           "-hmapWidth  int\n"
           "-hmapHeight int   rozmer ci rozliseni vyskove mapy\n"
           "-wsize float\n"
           "-hsize float      sirka a delka krajiny (v prostorovych souradnicich)\n"
           "-numIter int      pocet iteraci fault formation algoritmu\n"
           "-maxDelta float\n"
           "-minDelta float   vyska prvniho a posledniho zlomu v krajine\n"
           "-filterQ float    parametr filtrovaci funkce\n"
           "-hq int           hodnota 0,1 a 2 meni rezim tesselace krajiny\n"
           "                  (0 standard, 1 adaptivni smer hrany, 2 ctyri trojuhelniky)\n"
           "Priklad pouziti (se standardnimi hodnotami parametru):\n"
           "%s -seed 10 -hmapWidth 65 -hmapHeight 65 -wsize 64 -hsize 64\n"
           "-numIter 150 -maxDelta 2 -minDelta 0.05 -filterQ 0.85 -hq 0\n",
           argv[0], argv[0]);
#endif
    exit(EXIT_SUCCESS);
  }

  // Inicializace genertoru nhodnch sel
  int seed;
  if (getParam<int>(argc, argv, i, "-seed", seed))
    srand(seed);
  else
    srand((unsigned int)time(NULL));

  // Vytvo koen scny
  SoSeparator *root = new SoSeparator;
  root->ref();

  // Nastav intenzitu globlnho svtla ve scn
  SoEnvironment *env = new SoEnvironment;
  env->ambientIntensity.setValue(0.5);
  root->addChild(env);

  // Svtlo
/*  SoDirectionalLight *sunLight = new SoDirectionalLight;
  sunLight->direction.setValue(0.35f, -1.0f, 0.25f);
  sunLight->intensity.setValue(1.0f);
  root->addChild(sunLight);
*/
  // Model krajiny
  int hmapWidth  = DEFAULT_TERRAIN_HMAP_WIDTH;
  int hmapHeight = DEFAULT_TERRAIN_HMAP_HEIGHT;
  float wsize = DEFAULT_TERRAIN_WSIZE;
  float hsize = DEFAULT_TERRAIN_HSIZE;
  int NumIter    = DEFAULT_TERRAIN_NUM_ITER;
  float MaxDelta = DEFAULT_TERRAIN_MAX_DELTA;
  float MinDelta = DEFAULT_TERRAIN_MIN_DELTA;
  float FilterValue = DEFAULT_TERRAIN_FILTER_Q;
  int hq = DEFAULT_TERRAIN_HQ;

  getParam<int>(argc, argv, i, "-hmapWidth", hmapWidth);
  getParam<int>(argc, argv, i, "-hmapHeight", hmapHeight);
  getParam<float>(argc, argv, i, "-wsize", wsize);
  getParam<float>(argc, argv, i, "-hsize", hsize);
  getParam<int>(argc, argv, i, "-numIter", NumIter);
  getParam<float>(argc, argv, i, "-maxDelta", MaxDelta);
  getParam<float>(argc, argv, i, "-minDelta", MinDelta);
  getParam<float>(argc, argv, i, "-filterQ", FilterValue);
  getParam<int>(argc, argv, i, "-hq", hq);

  // Zkontroluj, zda byly zparsovny spn vechny argumenty z command-line.
  if (i != argc) {
#if !defined(__WIN32__) && !defined(_WIN32)
    printf("Chybn parametr %s.\n", argv[i]);
#else
    printf("Chybny parametr %s.\n", argv[i]);
#endif
    exit(EXIT_FAILURE);
  }

  // Zkontroluj platnost parametr danch uivatelem.
  // filterQ mus bt v rozsahu od 0.f do 1.f
  if (FilterValue < 0.f || FilterValue > 1.f) {
#if !defined(__WIN32__) && !defined(_WIN32)
    printf("Parametr filterQ mus bt v rozsahu 0.0 a 1.0.\n"
           "Pklad: 0.5, 0.36 a podobn.\n");
#else
    printf("Parametr filterQ musi byt v rozsahu 0.0 az 1.0.\n"
           "Priklad: 0.5, 0.36 a podobne.\n");
#endif
    exit(EXIT_FAILURE);
  }
  // Omez hq na hodnoty 0,1 a 2
  if (hq < 0 || hq > 2)  hq = 2;

  // Generovn krajiny
  printf("\nGeneruji krajinu fault formation algoritmem:\n"
         "hmap dimensions:  %ix%i\n"
         "size:             %.1fx%.1f\n"
         "numIter:          %i\n"
         "maxDelta:         %.2f\n"
         "minDelta:         %.2f\n"
         "filterQ:          %.2f\n"
         "hq:               %i\n\n",
         hmapWidth, hmapHeight, wsize, hsize,
         NumIter, MaxDelta, MinDelta, FilterValue, hq); 

  // my code modif -----------------------------------

  // vytvoen vkov mapy (pole float)
  terrain.generateHeightMap(hmapWidth, hmapHeight, NumIter, MaxDelta, MinDelta, FilterValue, wsize, hsize);

  // urcim vyskove urovne vody skal a snehu
  terrain.getLeveL(wsize, hsize, hq);
  
  // generuji mesto navazanim k root-u uzel town ktery vytvori fce.GenerTown
  // vse portebne ji predam (vysku ve vsech bodech krajiny a urovne vody skal a snehu)
  root->addChild(GenerTown(&terrain));

  // generuji scnu (model krajiny a modely budov)
  root->addChild(terrain.getModel(wsize, hsize, hq));
  
  // end my code -------------------------------------

  root->unrefNoDelete();
  return root;
}

