#include"buildings.h"

// OBELISK
// prjmen.prom obel_vertices...
static float vvertices[][3] =
{
	-4, 0, -4, // 0 
	 4, 0, -4, // 1
	 4, 0, 4,  // 2
	-4, 0, 4,  // 3

	 2, 27, 2,  // 4
     2, 27, -2, // 5
	-2, 27, -2, // 6
	-2, 27, 2,  // 7

	 0, 30, 0  // 8 

};

static float nnorms[][3] =
{
	0, .555f, .832f,    // 0 front_up
	-.832f, .555f, 0,   // 1 left_up
    0, .555f, -.832f,   // 2 back_up
	.832f, .555f, 0,    // 3 right_up
 
	0, .0739f, .9973f,  // 4 front
	-.9972f, .0739f, 0, // 5 left
	0, .0739f, -.9973f, // 6 back
	.9972f, .0739f, 0   // 7 right
};

static int numvertices_front_up[] = { 6,5,8, -1};
static int numvertices_back_up[] = { 7,4,8, -1};
static int numvertices_left_up[] = { 5,4,8, -1};
static int numvertices_right_up[] = { 7,6,8, -1};
static int numvertices_front[] = { 1,0,5,6, -1 };
static int numvertices_back[] = { 2,3,4,7, -1};
static int numvertices_left[] = { 2,1,4,5, -1};
static int numvertices_right[] = { 3,0,7,6, -1};

static int normalIndex_front_up[] = { 2 };
static int normalIndex_back_up[] = { 0 };
static int normalIndex_left_up[] = { 3 };
static int normalIndex_right_up[] = { 1 };
static int normalIndex_front[] = { 6,6 };
static int normalIndex_back[] = { 4,4 };
static int normalIndex_left[] = { 7,7 };
static int normalIndex_right[] = { 5,5 };

static int32_t TexCoordIndex_front_up[] = { 0,1,4, -1 };
static int32_t TexCoordIndex_back_up[] = { 0,1,4, -1 };
static int32_t TexCoordIndex_left_up[] = { 0,1,4, -1 };
static int32_t TexCoordIndex_right_up[] = { 0,1,4, -1 };
static int32_t TexCoordIndex_front[] = { 1,0,3,2, -1 };
static int32_t TexCoordIndex_back[] = { 1,0,3,2, -1 };
static int32_t TexCoordIndex_left[] = { 1,0,3,2, -1 };
static int32_t TexCoordIndex_right[] = { 1,0,3,2, -1 };


SoSeparator *makeObeliskFaceSet(float x, float y, float z)
{
	SoSeparator *obelisk = new SoSeparator();
	obelisk -> ref();

  SoTransform *transform = new SoTransform;
  transform->translation.setValue(x,y,z);
  obelisk->addChild(transform);

   SoScale *scale = new SoScale;
  scale->scaleFactor.setValue(.375f,.375f,.375f);
  obelisk->addChild(scale);

  SoNormal *myNormals = new SoNormal;
  myNormals -> vector.setValues(0, 8, nnorms);
  obelisk -> addChild(myNormals); 

  SoNormalBinding *myNormalBinding = new SoNormalBinding;
  myNormalBinding -> value = SoNormalBinding::PER_FACE_INDEXED;
  obelisk -> addChild(myNormalBinding);

	SoCoordinate3 *myCoords = new SoCoordinate3;
	myCoords -> point.setValues(0, sizeof(vvertices)/sizeof(float), vvertices);
	obelisk -> addChild(myCoords);

  SoTextureCoordinate2 *texCoord = new SoTextureCoordinate2;
  texCoord->point.set1Value(0, SbVec2f(0,0));
  texCoord->point.set1Value(1, SbVec2f(1,0));
  texCoord->point.set1Value(2, SbVec2f(0.25f,1));
  texCoord->point.set1Value(3, SbVec2f(0.75f,1));
  texCoord->point.set1Value(4, SbVec2f(0.5,1));
  obelisk->addChild(texCoord);


  // front_up
  SoTexture2 *texture = new SoTexture2;
  obelisk -> addChild(texture);
  texture -> filename.setValue("texture_front_up.PNG");

  SoIndexedTriangleStripSet *stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(numvertices_front_up)/sizeof(int32_t), numvertices_front_up);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_front_up)/sizeof(int), normalIndex_front_up);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_front_up)/sizeof(int32_t), TexCoordIndex_front_up); 
  obelisk->addChild(stripSet);


  // back_up
  texture = new SoTexture2;
  obelisk -> addChild(texture);
  texture -> filename.setValue("texture_back_up.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(numvertices_back_up)/sizeof(int32_t), numvertices_back_up);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_back_up)/sizeof(int), normalIndex_back_up);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_back_up)/sizeof(int32_t), TexCoordIndex_back_up); 
  obelisk->addChild(stripSet);


  // left_up
  texture = new SoTexture2;
  obelisk -> addChild(texture);
  texture -> filename.setValue("texture_left_up.PNG");

  // left_up
  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(numvertices_left_up)/sizeof(int32_t), numvertices_left_up);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_left_up)/sizeof(int), normalIndex_left_up);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_left_up)/sizeof(int32_t), TexCoordIndex_left_up); 
  obelisk->addChild(stripSet);


  // right_up
  texture = new SoTexture2;
  obelisk -> addChild(texture);
  texture -> filename.setValue("texture_right_up.PNG");

  // right_up
  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(numvertices_right_up)/sizeof(int32_t), numvertices_right_up);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_right_up)/sizeof(int), normalIndex_right_up);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_right_up)/sizeof(int32_t), TexCoordIndex_right_up); 
  obelisk->addChild(stripSet);


  // front
  texture = new SoTexture2;
  obelisk -> addChild(texture);
  texture -> filename.setValue("texture_front.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(numvertices_front)/sizeof(int32_t), numvertices_front);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_front)/sizeof(int), normalIndex_front);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_front)/sizeof(int32_t), TexCoordIndex_front); 
  obelisk->addChild(stripSet);


  // back
  texture = new SoTexture2;
  obelisk -> addChild(texture);
  texture -> filename.setValue("texture_back.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(numvertices_back)/sizeof(int32_t), numvertices_back);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_back)/sizeof(int), normalIndex_back);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_back)/sizeof(int32_t), TexCoordIndex_back); 
  obelisk->addChild(stripSet);


  // left
  texture = new SoTexture2;
  obelisk -> addChild(texture);
  texture -> filename.setValue("texture_left.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(numvertices_left)/sizeof(int32_t), numvertices_left);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_left)/sizeof(int), normalIndex_left);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_left)/sizeof(int32_t), TexCoordIndex_left); 
  obelisk->addChild(stripSet);


  // right
  texture = new SoTexture2;
  obelisk -> addChild(texture);
  texture -> filename.setValue("texture_right.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(numvertices_right)/sizeof(int32_t), numvertices_right);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_right)/sizeof(int), normalIndex_right);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_right)/sizeof(int32_t), TexCoordIndex_right); 
  obelisk->addChild(stripSet);

	obelisk -> unrefNoDelete();
	return obelisk;
}