#include"buildings.h"

// VERTICES 
// body v prostoru jejich propojenim vytvarim model budovy 

static float p10_vertices[][3] = {
	-1.5f, 0, -1,  // 0
 	 1.5f, 0, -1,  // 1
    -1.5f, 2, -1,  // 2
     1.5f, 2, -1,  // 3

	-1.5f, 0, 0,  // 4
     0.5f, 0, 0,  // 5
    -1.5f, 2, 0,  // 6
     0.5f, 2, 0,  // 7

	 0.5f, 0, 1,  // 8
     1.5f, 0, 1,  // 9
     0.5f, 2, 1,  // 10
     1.5f, 2, 1   // 11
};


// NORMALS
// normaly pro dobre osvetleni sten modelu budovy

static float p10_normal[][3] = {

	1, 0, 0, // 0 x 
    0, 1, 0, // 1 y
    0, 0, 1, // 2 z
 
	-1, 0, 0, // 3 -x
    0, -1, 0, // 4 -y
    0, 0, -1, // 5 -z
	
};

static int32_t p10_indices_front[] = {0,1,2,3, -1};
static int32_t p10_indices_left[] = {0,2,4,6, -1};
static int32_t p10_indices_right_front[] = {1,3,9,11, -1};
static int32_t p10_indices_right_back[] = {5,7,8,10, -1};
static int32_t p10_indices_back_near[] = {4,5,6,7, -1};
static int32_t p10_indices_back_far[] = {8,9,10,11, -1};
static int32_t p10_indices_up[] = {2,6,3,7,11,10, -1};
static int32_t p10_indices_bottom[] = {0,4,1,5,9,8, -1};

static int normalIndex_p10_front[] = {5,5};   // -z     
static int normalIndex_p10_left[] = {3,3};    // -x
static int normalIndex_p10_right_front[] = {0,0};   //  x
static int normalIndex_p10_right_back[] = {3,3};   //  x
static int normalIndex_p10_back_near[] = {2,2};    //  z
static int normalIndex_p10_back_far[] = {2,2};    //  z
static int normalIndex_p10_up[] = {1,1,1,1};      //  y  
static int normalIndex_p10_bottom[] = {4,4,4,4};  // -y

static int32_t TexCoordIndex_p10_front[] = {0,1,2,3, -1};
static int32_t TexCoordIndex_p10_up[] = {0,4,1,5,3,6, -1};
static int32_t TexCoordIndex_p10_left[] = {0,2,1,3, -1};
static int32_t TexCoordIndex_p10_right_front[] = {0,2,1,3, -1};
static int32_t TexCoordIndex_p10_right_back[] = {0,2,1,3, -1};
static int32_t TexCoordIndex_p10_back_near[] = {0,1,2,3, -1};
static int32_t TexCoordIndex_p10_back_far[] = {0,1,2,3, -1};
static int32_t TexCoordIndex_p10_bottom[] = {0,4,1,5,3,6, -1};

// PANELAK 1
// krychle ( cube pres FaceSet :) ) 
SoSeparator *make_p10_FaceSet(float x, float y, float z, float alfa)
{
  // 
  SoSeparator *p10 = new SoSeparator;
  p10 -> ref();

  // umisteni budovy
  SoTransform *transform = new SoTransform;
  transform->translation.setValue(x,y,z);
  p10->addChild(transform);

  // velikost modelu budovy 
  SoScale *scale = new SoScale;
  scale->scaleFactor.setValue(SCALE_X,SCALE_Y,SCALE_Z);
  p10->addChild(scale);

  // rotace modelu budovy 
  SoRotationXYZ *myRotation = new SoRotationXYZ;     
  myRotation -> angle = alfa;
  myRotation -> axis = SoRotationXYZ::Y;
  p10 -> addChild(myRotation);

  // normaly modelu budovy
  SoNormal *myNormals = new SoNormal;
  myNormals -> vector.setValues(0, 10, p10_normal);
  p10 -> addChild(myNormals); 

  // 
  SoNormalBinding *myNormalBinding = new SoNormalBinding;
  myNormalBinding -> value = SoNormalBinding::PER_FACE_INDEXED;
  p10 -> addChild(myNormalBinding);

  // propojeni bodu modelu budovy 
  SoCoordinate3 *coords = new SoCoordinate3;
  coords->point.setValues(0, sizeof(p10_vertices)/sizeof(float), p10_vertices);
  p10->addChild(coords);

  // 'vyrez textury (tvar 2D pro krychly potrebujeme ctverec bereme celou textuturu)' 
  SoTextureCoordinate2 *texCoord = new SoTextureCoordinate2;
  texCoord->point.set1Value(0, SbVec2f(0,0));
  texCoord->point.set1Value(1, SbVec2f(1,0));
  texCoord->point.set1Value(2, SbVec2f(0,1));
  texCoord->point.set1Value(3, SbVec2f(1,1));

  texCoord->point.set1Value(4, SbVec2f(0,0.5));
  texCoord->point.set1Value(5, SbVec2f(0.6666f,0.5));
  texCoord->point.set1Value(6, SbVec2f(0.6666f,1));
  p10->addChild(texCoord);

  // front
  SoTexture2 *texture_p10 = new SoTexture2;
  p10 -> addChild(texture_p10);
  texture_p10 -> filename.setValue("Texture/p10/texture_p10_front.PNG");

  SoIndexedTriangleStripSet *stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(p10_indices_front)/sizeof(int32_t), p10_indices_front);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_p10_front)/sizeof(int), normalIndex_p10_front);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_p10_front)/sizeof(int32_t), TexCoordIndex_p10_front); 
  p10->addChild(stripSet);


  // up
  texture_p10 = new SoTexture2;
  p10 -> addChild(texture_p10);
  texture_p10 -> filename.setValue("Texture/p10/texture_p10_up.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(p10_indices_up)/sizeof(int32_t), p10_indices_up);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_p10_up)/sizeof(int), normalIndex_p10_up);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_p10_up)/sizeof(int32_t), TexCoordIndex_p10_up); 
  p10->addChild(stripSet);


  // left
  texture_p10 = new SoTexture2;
  p10 -> addChild(texture_p10);
  texture_p10 -> filename.setValue("Texture/p10/texture_p10_left.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(p10_indices_left)/sizeof(int32_t), p10_indices_left);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_p10_left)/sizeof(int), normalIndex_p10_left);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_p10_left)/sizeof(int32_t), TexCoordIndex_p10_left); 
  p10->addChild(stripSet);


  // right_front
  texture_p10 = new SoTexture2;
  p10 -> addChild(texture_p10);
  texture_p10 -> filename.setValue("Texture/p10/texture_p10_right_front.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(p10_indices_right_front)/sizeof(int32_t), p10_indices_right_front);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_p10_right_front)/sizeof(int), normalIndex_p10_right_front);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_p10_right_front)/sizeof(int32_t), TexCoordIndex_p10_right_front); 
  p10->addChild(stripSet);

  // right_back
  texture_p10 = new SoTexture2;
  p10 -> addChild(texture_p10);
  texture_p10 -> filename.setValue("Texture/p10/texture_p10_right_back.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(p10_indices_right_back)/sizeof(int32_t), p10_indices_right_back);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_p10_right_back)/sizeof(int), normalIndex_p10_right_back);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_p10_right_back)/sizeof(int32_t), TexCoordIndex_p10_right_back); 
  p10->addChild(stripSet);

  
  // back_near
  texture_p10 = new SoTexture2;
  p10 -> addChild(texture_p10);
  texture_p10 -> filename.setValue("Texture/p10/texture_p10_back_near.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(p10_indices_back_near)/sizeof(int32_t), p10_indices_back_near);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_p10_back_near)/sizeof(int), normalIndex_p10_back_near);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_p10_back_near)/sizeof(int32_t), TexCoordIndex_p10_back_near); 
  p10->addChild(stripSet);

  // back_far
  texture_p10 = new SoTexture2;
  p10 -> addChild(texture_p10);
  texture_p10 -> filename.setValue("Texture/p10/texture_p10_back_far.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(p10_indices_back_far)/sizeof(int32_t), p10_indices_back_far);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_p10_back_far)/sizeof(int), normalIndex_p10_back_far);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_p10_back_far)/sizeof(int32_t), TexCoordIndex_p10_back_far); 
  p10->addChild(stripSet);


  // bottom
  texture_p10 = new SoTexture2;
  p10 -> addChild(texture_p10);
  texture_p10 -> filename.setValue("Texture/p10/texture_bottom.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(p10_indices_bottom)/sizeof(int32_t), p10_indices_bottom);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_p10_bottom)/sizeof(int), normalIndex_p10_bottom);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_p10_bottom)/sizeof(int32_t), TexCoordIndex_p10_bottom); 
  p10->addChild(stripSet);


  p10 -> unrefNoDelete();

  return p10;
}
// end PANELAK 1