#include"buildings.h"

// VERTICES 
// body v prostoru jejich propojenim vytvarim model budovy 

static float p11_vertices[][3] = {
	-2.5f, 0, -1,  // 0
	 2.5f, 0, -1,  // 1
    -2.5f, 3, -1,  // 2
     1, 3, -1,  // 3
     1, 2, -1,  // 4
     2.5f, 2, -1,  // 5

	-2.5f, 0, 1,  // 6
     2.5f, 0, 1,  // 7
    -2.5f, 3, 1,  // 8
     1, 3, 1,  // 9
     1, 2, 1,  // 10
     2.5f, 2, 1,  // 11
};


// NORMALS
// normaly pro dobre osvetleni sten modelu budovy

static float p11_normal[][3] = {

	1, 0, 0, // 0 x 
    0, 1, 0, // 1 y
    0, 0, 1, // 2 z
 
	-1, 0, 0, // 3 -x
    0, -1, 0, // 4 -y
    0, 0, -1, // 5 -z
	
};

static int32_t p11_indices_front[] = {2,3,0,4,1,5, -1};
static int32_t p11_indices_left[] = {0,2,6,8, -1};
static int32_t p11_indices_right_near[] = {1,5,7,11, -1};
static int32_t p11_indices_right_far[] = {3,4,9,10, -1};
static int32_t p11_indices_back[] = {8,9,6,10,7,11, -1};
static int32_t p11_indices_up_high[] = {2,3,8,9, -1};
static int32_t p11_indices_up_low[] = {4,5,10,11, -1};
static int32_t p11_indices_bottom[] = {0,1,6,7, -1};

static int normalIndex_p11_front[] = {5,5,5,5};   // -z     
static int normalIndex_p11_left[] = {3,3};    // -x
static int normalIndex_p11_right_near[] = {0,0};   //  x
static int normalIndex_p11_right_far[] = {0,0};   //  x
static int normalIndex_p11_back[] = {2,2,2,2};    //  z
static int normalIndex_p11_up_high[] = {1,1};      //  y 
static int normalIndex_p11_up_low[] = {1,1};      //  y 
static int normalIndex_p11_bottom[] = {4,4};  // -y

static int32_t TexCoordIndex_p11_front[] = {2,4,0,5,1,6, -1};
static int32_t TexCoordIndex_p11_up_high[] = {0,1,2,3, -1};
static int32_t TexCoordIndex_p11_up_low[] = {0,1,2,3, -1};
static int32_t TexCoordIndex_p11_left[] = {0,2,1,3, -1};
static int32_t TexCoordIndex_p11_right_near[] = {0,2,1,3, -1};
static int32_t TexCoordIndex_p11_right_far[] = {0,2,1,3, -1};
static int32_t TexCoordIndex_p11_back[] = {2,4,0,5,1,6, -1};
static int32_t TexCoordIndex_p11_bottom[] = {0,1,2,3, -1};

// PANELAK 1
// krychle ( cube pres FaceSet :) ) 
SoSeparator *make_p11_FaceSet(float x, float y, float z, float alfa)
{
  // 
  SoSeparator *p11 = new SoSeparator;
  p11 -> ref();

  // umisteni budovy
  SoTransform *transform = new SoTransform;
  transform->translation.setValue(x,y,z);
  p11->addChild(transform);

  // velikost modelu budovy 
  SoScale *scale = new SoScale;
  scale->scaleFactor.setValue(SCALE_X,SCALE_Y,SCALE_Z);
  p11->addChild(scale);

  // rotace modelu budovy 
  SoRotationXYZ *myRotation = new SoRotationXYZ;     
  myRotation -> angle = alfa;
  myRotation -> axis = SoRotationXYZ::Y;
  p11 -> addChild(myRotation);

  // normaly modelu budovy
  SoNormal *myNormals = new SoNormal;
  myNormals -> vector.setValues(0, 10, p11_normal);
  p11 -> addChild(myNormals); 

  // 
  SoNormalBinding *myNormalBinding = new SoNormalBinding;
  myNormalBinding -> value = SoNormalBinding::PER_FACE_INDEXED;
  p11 -> addChild(myNormalBinding);

  // propojeni bodu modelu budovy 
  SoCoordinate3 *coords = new SoCoordinate3;
  coords->point.setValues(0, sizeof(p11_vertices)/sizeof(float), p11_vertices);
  p11->addChild(coords);

  // 'vyrez textury (tvar 2D pro krychly potrebujeme ctverec bereme celou textuturu)' 
  SoTextureCoordinate2 *texCoord = new SoTextureCoordinate2;
  texCoord->point.set1Value(0, SbVec2f(0,0));
  texCoord->point.set1Value(1, SbVec2f(1,0));
  texCoord->point.set1Value(2, SbVec2f(0,1));
  texCoord->point.set1Value(3, SbVec2f(1,1));

  texCoord->point.set1Value(4, SbVec2f(0.7f,1)); // 3 9
  texCoord->point.set1Value(5, SbVec2f(0.7f,0.6666f)); // 4 10
  texCoord->point.set1Value(6, SbVec2f(1,0.6666f)); // 5 11
  p11->addChild(texCoord);

  // front
  SoTexture2 *texture_p11 = new SoTexture2;
  p11 -> addChild(texture_p11);
  texture_p11 -> filename.setValue("Texture/p11/texture_p11_front.PNG");

  SoIndexedTriangleStripSet *stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(p11_indices_front)/sizeof(int32_t), p11_indices_front);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_p11_front)/sizeof(int), normalIndex_p11_front);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_p11_front)/sizeof(int32_t), TexCoordIndex_p11_front); 
  p11->addChild(stripSet);


  // up_high
  texture_p11 = new SoTexture2;
  p11 -> addChild(texture_p11);
  texture_p11 -> filename.setValue("Texture/p11/texture_p11_up_high.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(p11_indices_up_high)/sizeof(int32_t), p11_indices_up_high);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_p11_up_high)/sizeof(int), normalIndex_p11_up_high);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_p11_up_high)/sizeof(int32_t), TexCoordIndex_p11_up_high); 
  p11->addChild(stripSet);

  // up_low
  texture_p11 = new SoTexture2;
  p11 -> addChild(texture_p11);
  texture_p11 -> filename.setValue("Texture/p11/texture_p11_up_low.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(p11_indices_up_low)/sizeof(int32_t), p11_indices_up_low);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_p11_up_low)/sizeof(int), normalIndex_p11_up_low);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_p11_up_low)/sizeof(int32_t), TexCoordIndex_p11_up_low); 
  p11->addChild(stripSet);


  // left
  texture_p11 = new SoTexture2;
  p11 -> addChild(texture_p11);
  texture_p11 -> filename.setValue("Texture/p11/texture_p11_left.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(p11_indices_left)/sizeof(int32_t), p11_indices_left);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_p11_left)/sizeof(int), normalIndex_p11_left);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_p11_left)/sizeof(int32_t), TexCoordIndex_p11_left); 
  p11->addChild(stripSet);


  // right_near
  texture_p11 = new SoTexture2;
  p11 -> addChild(texture_p11);
  texture_p11 -> filename.setValue("Texture/p11/texture_p11_right_down.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(p11_indices_right_near)/sizeof(int32_t), p11_indices_right_near);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_p11_right_near)/sizeof(int), normalIndex_p11_right_near);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_p11_right_near)/sizeof(int32_t), TexCoordIndex_p11_right_near); 
  p11->addChild(stripSet);

  // right_far
  texture_p11 = new SoTexture2;
  p11 -> addChild(texture_p11);
  texture_p11 -> filename.setValue("Texture/p11/texture_p11_right_up.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(p11_indices_right_far)/sizeof(int32_t), p11_indices_right_far);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_p11_right_far)/sizeof(int), normalIndex_p11_right_far);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_p11_right_far)/sizeof(int32_t), TexCoordIndex_p11_right_far); 
  p11->addChild(stripSet);

  
  // back
  texture_p11 = new SoTexture2;
  p11 -> addChild(texture_p11);
  texture_p11 -> filename.setValue("Texture/p11/texture_p11_back.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(p11_indices_back)/sizeof(int32_t), p11_indices_back);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_p11_back)/sizeof(int), normalIndex_p11_back);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_p11_back)/sizeof(int32_t), TexCoordIndex_p11_back); 
  p11->addChild(stripSet);


  // bottom
  texture_p11 = new SoTexture2;
  p11 -> addChild(texture_p11);
  texture_p11 -> filename.setValue("Texture/p11/texture_bottom.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(p11_indices_bottom)/sizeof(int32_t), p11_indices_bottom);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_p11_bottom)/sizeof(int), normalIndex_p11_bottom);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_p11_bottom)/sizeof(int32_t), TexCoordIndex_p11_bottom); 
  p11->addChild(stripSet);

  p11 -> unrefNoDelete();

  return p11;
}
// end PANELAK 1