#include"buildings.h"

// VERTICES 
// body v prostoru jejich propojenim vytvarim model budovy 

static float s11_vertices[][3] = {
	-1.5f, 0, -1,  // 0
	 1.5f, 0, -1,  // 1
    -1.5f, 2, -1,  // 2
     1.5f, 2, -1,  // 3

	-1.5f, 0, 0,  // 4
     0.5f, 0, 0,  // 5
    -1.5f, 2, 0,  // 6
     0.5f, 2, 0,  // 7

	 0.5f, 0, 1,  // 8
     1.5f, 0, 1,  // 9
     0.5f, 2, 1,  // 10
     1.5f, 2, 1,  // 11

	-1, 2.5, -0.5f,  // 12
	 1, 2.5, -0.5f,  // 13
     1, 2.5,  0.5f   // 14
};


// NORMALS
// normaly pro dobre osvetleni sten modelu budovy

static float s11_normal[][3] = {

	1, 0, 0, // 0 x 
    0, 1, 0, // 1 y
    0, 0, 1, // 2 z
 
	-1, 0, 0, // 3 -x
    0, -1, 0, // 4 -y
    0, 0, -1, // 5 -z

	 0.70710f,0.70710f,0, // 6 left_up 45 sklon strechy 
	-0.70710f,0.70710f,0, // 7 right_up 45 sklon strechy 
	 0,0.70710f,0.70710f, // 8 back_up 45 sklon strechy 
	 0,0.70710f,-0.70710f,// 9 front_up 45 sklon strechy 
};

static int32_t s11_indices_front[] = {0,1,2,3, -1};
static int32_t s11_indices_left[] = {0,2,4,6, -1};
static int32_t s11_indices_right_front[] = {1,3,9,11, -1};
static int32_t s11_indices_right_back[] = {5,7,8,10, -1};
static int32_t s11_indices_back_near[] = {4,5,6,7, -1};
static int32_t s11_indices_back_far[] = {8,9,10,11, -1};
static int32_t s11_indices_up_first[] = {2,3,12,13, -1};
static int32_t s11_indices_up_second[] = {6,7,12,13, -1};
static int32_t s11_indices_up_third[] = {3,11,13,14, -1};
static int32_t s11_indices_up_fourth[] = {7,10,13,14, -1};
static int32_t s11_indices_up_fifth[] = {2,6,12, -1};
static int32_t s11_indices_up_sixth[] = {10,11,14, -1};
static int32_t s11_indices_bottom[] = {0,4,1,5,9,8, -1};

static int normalIndex_s11_front[] = {5,5};   // -z     
static int normalIndex_s11_left[] = {3,3};    // -x
static int normalIndex_s11_right_front[] = {0,0};   //  x
static int normalIndex_s11_right_back[] = {3,3};   //  x
static int normalIndex_s11_back_near[] = {2,2};    //  z
static int normalIndex_s11_back_far[] = {2,2};    //  z
static int normalIndex_s11_up_first[] = {9,9};  //  sklon strechy 
static int normalIndex_s11_up_second[] = {8,8};  //  sklon strechy
static int normalIndex_s11_up_third[] = {6,6};  //  sklon strechy
static int normalIndex_s11_up_fourth[] = {7,7};  //  sklon strechy
static int normalIndex_s11_up_fifth[] = {7};  //  sklon strechy
static int normalIndex_s11_up_sixth[] = {8};  //  sklon strechy
static int normalIndex_s11_bottom[] = {4,4,4,4};  // -y

static int32_t TexCoordIndex_s11_front[] = {0,1,2,3, -1};
static int32_t TexCoordIndex_s11_up_first[] = {0,1,7,8, -1};
static int32_t TexCoordIndex_s11_up_second[] = {2,10,9,1, -1};
static int32_t TexCoordIndex_s11_up_third[] = {0,1,11,12, -1};
static int32_t TexCoordIndex_s11_up_fourth[] = {13,1,2,14, -1};
static int32_t TexCoordIndex_s11_up_fifth[] = {0,1,4, -1};
static int32_t TexCoordIndex_s11_up_sixth[] = {0,1,4, -1};
static int32_t TexCoordIndex_s11_left[] = {0,2,1,3, -1};
static int32_t TexCoordIndex_s11_right_front[] = {0,2,1,3, -1};
static int32_t TexCoordIndex_s11_right_back[] = {0,2,1,3, -1};
static int32_t TexCoordIndex_s11_back_near[] = {0,1,2,3, -1};
static int32_t TexCoordIndex_s11_back_far[] = {0,1,2,3, -1};
static int32_t TexCoordIndex_s11_bottom[] = {0,4,1,5,3,6, -1};

// PANELAK 1
// krychle ( cube pres FaceSet :) ) 
SoSeparator *make_s11_FaceSet(float x, float y, float z, float alfa)
{
  // 
  SoSeparator *s11 = new SoSeparator;
  s11 -> ref();

  // umisteni budovy
  SoTransform *transform = new SoTransform;
  transform->translation.setValue(x,y,z);
  s11->addChild(transform);

  // velikost modelu budovy 
  SoScale *scale = new SoScale;
  scale->scaleFactor.setValue(SCALE_X,SCALE_Y,SCALE_Z);
  s11->addChild(scale);

  // rotace modelu budovy
  SoRotationXYZ *myRotation = new SoRotationXYZ;     
  myRotation -> angle = alfa;
  myRotation -> axis = SoRotationXYZ::Y;
  s11 -> addChild(myRotation);

  // normaly modelu budovy
  SoNormal *myNormals = new SoNormal;
  myNormals -> vector.setValues(0, 10, s11_normal);
  s11 -> addChild(myNormals); 

  // 
  SoNormalBinding *myNormalBinding = new SoNormalBinding;
  myNormalBinding -> value = SoNormalBinding::PER_FACE_INDEXED;
  s11 -> addChild(myNormalBinding);

  // propojeni bodu modelu budovy 
  SoCoordinate3 *coords = new SoCoordinate3;
  coords->point.setValues(0, sizeof(s11_vertices)/sizeof(float), s11_vertices);
  s11->addChild(coords);

  // 'vyrez textury (tvar 2D pro krychly potrebujeme ctverec bereme celou textuturu)' 
  SoTextureCoordinate2 *texCoord = new SoTextureCoordinate2;
  texCoord->point.set1Value(0, SbVec2f(0,0));
  texCoord->point.set1Value(1, SbVec2f(1,0));
  texCoord->point.set1Value(2, SbVec2f(0,1));
  texCoord->point.set1Value(3, SbVec2f(1,1));

  texCoord->point.set1Value(4, SbVec2f(0.5f,1));
  texCoord->point.set1Value(5, SbVec2f(0.6666f,0.5));
  texCoord->point.set1Value(6, SbVec2f(0.6666f,1));

  texCoord->point.set1Value(7, SbVec2f(0.16666f,1));
  texCoord->point.set1Value(8, SbVec2f(0.83333f,1));

  texCoord->point.set1Value(9, SbVec2f(0.2f,0));
  texCoord->point.set1Value(10, SbVec2f(0.8f,1));

  texCoord->point.set1Value(11, SbVec2f(0.25f,1));
  texCoord->point.set1Value(12, SbVec2f(0.75f,1));

  texCoord->point.set1Value(13, SbVec2f(0.3333f,0));
  texCoord->point.set1Value(14, SbVec2f(0.6666f,1));

  s11->addChild(texCoord);

  // front
  SoTexture2 *texture_s11 = new SoTexture2;
  s11 -> addChild(texture_s11);
  texture_s11 -> filename.setValue("Texture/s11/texture_s11_front.PNG");

  SoIndexedTriangleStripSet *stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s11_indices_front)/sizeof(int32_t), s11_indices_front);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s11_front)/sizeof(int), normalIndex_s11_front);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s11_front)/sizeof(int32_t), TexCoordIndex_s11_front); 
  s11->addChild(stripSet);


  // up_first
  texture_s11 = new SoTexture2;
  s11 -> addChild(texture_s11);
  texture_s11 -> filename.setValue("Texture/s11/texture_s11_up_front.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s11_indices_up_first)/sizeof(int32_t), s11_indices_up_first);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s11_up_first)/sizeof(int), normalIndex_s11_up_first);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s11_up_first)/sizeof(int32_t), TexCoordIndex_s11_up_first); 
  s11->addChild(stripSet);

  // up_second
  texture_s11 = new SoTexture2;
  s11 -> addChild(texture_s11);
  texture_s11 -> filename.setValue("Texture/s11/texture_s11_up_back.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s11_indices_up_second)/sizeof(int32_t), s11_indices_up_second);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s11_up_second)/sizeof(int), normalIndex_s11_up_second);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s11_up_second)/sizeof(int32_t), TexCoordIndex_s11_up_second); 
  s11->addChild(stripSet);

  // up_third
  texture_s11 = new SoTexture2;
  s11 -> addChild(texture_s11);
  texture_s11 -> filename.setValue("Texture/s11/texture_s11_up_right.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s11_indices_up_third)/sizeof(int32_t), s11_indices_up_third);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s11_up_third)/sizeof(int), normalIndex_s11_up_third);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s11_up_third)/sizeof(int32_t), TexCoordIndex_s11_up_third); 
  s11->addChild(stripSet);

  // up_fourth
  texture_s11 = new SoTexture2;
  s11 -> addChild(texture_s11);
  texture_s11 -> filename.setValue("Texture/s11/texture_s11_up_left.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s11_indices_up_fourth)/sizeof(int32_t), s11_indices_up_fourth);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s11_up_fourth)/sizeof(int), normalIndex_s11_up_fourth);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s11_up_fourth)/sizeof(int32_t), TexCoordIndex_s11_up_fourth); 
  s11->addChild(stripSet);

  // up_fifth
  texture_s11 = new SoTexture2;
  s11 -> addChild(texture_s11);
  texture_s11 -> filename.setValue("Texture/s11/texture_s11_up.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s11_indices_up_fifth)/sizeof(int32_t), s11_indices_up_fifth);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s11_up_fifth)/sizeof(int), normalIndex_s11_up_fifth);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s11_up_fifth)/sizeof(int32_t), TexCoordIndex_s11_up_fifth); 
  s11->addChild(stripSet);

  // up_sixth
  texture_s11 = new SoTexture2;
  s11 -> addChild(texture_s11);
  texture_s11 -> filename.setValue("Texture/s11/texture_s11_up.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s11_indices_up_sixth)/sizeof(int32_t), s11_indices_up_sixth);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s11_up_sixth)/sizeof(int), normalIndex_s11_up_sixth);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s11_up_sixth)/sizeof(int32_t), TexCoordIndex_s11_up_sixth); 
  s11->addChild(stripSet);

  // left
  texture_s11 = new SoTexture2;
  s11 -> addChild(texture_s11);
  texture_s11 -> filename.setValue("Texture/s11/texture_s11_left.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s11_indices_left)/sizeof(int32_t), s11_indices_left);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s11_left)/sizeof(int), normalIndex_s11_left);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s11_left)/sizeof(int32_t), TexCoordIndex_s11_left); 
  s11->addChild(stripSet);


  // right_front
  texture_s11 = new SoTexture2;
  s11 -> addChild(texture_s11);
  texture_s11 -> filename.setValue("Texture/s11/texture_s11_right.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s11_indices_right_front)/sizeof(int32_t), s11_indices_right_front);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s11_right_front)/sizeof(int), normalIndex_s11_right_front);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s11_right_front)/sizeof(int32_t), TexCoordIndex_s11_right_front); 
  s11->addChild(stripSet);

  // right_back
  texture_s11 = new SoTexture2;
  s11 -> addChild(texture_s11);
  texture_s11 -> filename.setValue("Texture/s11/texture_s11_left.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s11_indices_right_back)/sizeof(int32_t), s11_indices_right_back);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s11_right_back)/sizeof(int), normalIndex_s11_right_back);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s11_right_back)/sizeof(int32_t), TexCoordIndex_s11_right_back); 
  s11->addChild(stripSet);

  
  // back_near
  texture_s11 = new SoTexture2;
  s11 -> addChild(texture_s11);
  texture_s11 -> filename.setValue("Texture/s11/texture_s11_back.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s11_indices_back_near)/sizeof(int32_t), s11_indices_back_near);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s11_back_near)/sizeof(int), normalIndex_s11_back_near);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s11_back_near)/sizeof(int32_t), TexCoordIndex_s11_back_near); 
  s11->addChild(stripSet);

  // back_far
  texture_s11 = new SoTexture2;
  s11 -> addChild(texture_s11);
  texture_s11 -> filename.setValue("Texture/s11/texture_s11_left.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s11_indices_back_far)/sizeof(int32_t), s11_indices_back_far);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s11_back_far)/sizeof(int), normalIndex_s11_back_far);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s11_back_far)/sizeof(int32_t), TexCoordIndex_s11_back_far); 
  s11->addChild(stripSet);


  // bottom
  texture_s11 = new SoTexture2;
  s11 -> addChild(texture_s11);
  texture_s11 -> filename.setValue("Texture/s11/texture_bottom.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s11_indices_bottom)/sizeof(int32_t), s11_indices_bottom);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s11_bottom)/sizeof(int), normalIndex_s11_bottom);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s11_bottom)/sizeof(int32_t), TexCoordIndex_s11_bottom); 
  s11->addChild(stripSet);


  s11 -> unrefNoDelete();

  return s11;
}
// end PANELAK 1