#include"buildings.h"

static float s15_vertices[][3] = {
	-1, 0, -0.5f,  // 0
	1, 0, -0.5f,  // 1
    1, 1, -0.5f,  // 2
    -1, 1, -0.5f,  // 3

    -1, 1.5, 0.5f,  // 4
	1, 1.5, 0.5f,  // 5

	1, 1, 0.5f,  // 6
    -1, 1, 0.5f,  // 7
    -1, 0, 0.5f,  // 8
    1, 0, 0.5f,  // 9


};

static float s15_normal[][3] = {

	1, 0, 0, // 0 x 
    0, 1, 0, // 1 y
    0, 0, 1, // 2 z
 
	-1, 0, 0, // 3 -x
    0, -1, 0, // 4 -y
    0, 0, -1, // 5 -z

    0,0.89442f,-0.44721f  // 6 front_up sklon strechy
};

static int32_t s15_indices_front[] = { 1,0,2,3, -1 };
static int32_t s15_indices_up[] = { 2,3,5,4, -1 };
static int32_t s15_indices_left[] = { 5,2,6,1,9, -1 };
static int32_t s15_indices_right[] = { 4,3,7,0,8, -1 };
static int32_t s15_indices_back[] = { 4,5,8,9, -1 };
static int32_t s15_indices_bottom[] = { 0,1,8,9, -1 };

static int normalIndex_s15_front[] = { 5, 5 };
static int normalIndex_s15_up[] = { 6, 6 };
static int normalIndex_s15_left[] = { 0, 0, 0 };
static int normalIndex_s15_right[] = { 3, 3, 3 };
static int normalIndex_s15_back[] = { 2, 2 };
static int normalIndex_s15_bottom[] = { 4, 4 };

static int32_t TexCoordIndex_s15_front[] = { 0,1,2,3, -1 };
static int32_t TexCoordIndex_s15_up[] = { 0,1,2,3, -1 };
static int32_t TexCoordIndex_s15_left[] = { 3,4,5,0,1, -1 };
static int32_t TexCoordIndex_s15_right[] = { 3,4,5,0,1, -1 };
static int32_t TexCoordIndex_s15_back[] = { 2,3,0,1, -1 };
static int32_t TexCoordIndex_s15_bottom[] = { 0,1,2,3, -1 };


SoSeparator *make_s15_FaceSet(float x, float y, float z, float alfa)
{
	SoSeparator *s15 = new SoSeparator();
	s15 -> ref();

    SoTransform *transform = new SoTransform;
    transform->translation.setValue(x,y,z);
    s15->addChild(transform);

    SoScale *scale = new SoScale;
    scale->scaleFactor.setValue(SCALE_X,SCALE_Y,SCALE_Z);
    s15->addChild(scale);

  // rotace modelu budovy
  SoRotationXYZ *myRotation = new SoRotationXYZ;     
  myRotation -> angle = alfa;
  myRotation -> axis = SoRotationXYZ::Y;
  s15 -> addChild(myRotation);

  SoNormal *myNormals = new SoNormal;
  myNormals -> vector.setValues(0, 11, s15_normal);
  s15 -> addChild(myNormals); 

  SoNormalBinding *myNormalBinding = new SoNormalBinding;
  myNormalBinding -> value = SoNormalBinding::PER_FACE_INDEXED;
  s15 -> addChild(myNormalBinding);
  
  SoCoordinate3 *coords = new SoCoordinate3;
  coords->point.setValues(0, sizeof(s15_vertices)/sizeof(float), s15_vertices);
  s15->addChild(coords);

  SoTextureCoordinate2 *texCoord = new SoTextureCoordinate2;
  texCoord->point.set1Value(0, SbVec2f(0,0));
  texCoord->point.set1Value(1, SbVec2f(1,0));
  texCoord->point.set1Value(2, SbVec2f(0,1));
  texCoord->point.set1Value(3, SbVec2f(1,1));
  texCoord->point.set1Value(4, SbVec2f(0,0.6666f));
  texCoord->point.set1Value(5, SbVec2f(1,0.6666f));

  s15->addChild(texCoord);

  // front
  SoTexture2 *texture_s15 = new SoTexture2;
  s15 -> addChild(texture_s15);
  texture_s15 -> filename.setValue("Texture/s15/texture_s15_front.PNG");

  SoIndexedTriangleStripSet *stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s15_indices_front)/sizeof(int32_t), s15_indices_front);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s15_front)/sizeof(int), normalIndex_s15_front);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s15_front)/sizeof(int32_t), TexCoordIndex_s15_front); 
  s15->addChild(stripSet);


  // up
  texture_s15 = new SoTexture2;
  s15 -> addChild(texture_s15);
  texture_s15 -> filename.setValue("Texture/s15/texture_s15_up.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s15_indices_up)/sizeof(int32_t), s15_indices_up);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s15_up)/sizeof(int), normalIndex_s15_up);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s15_up)/sizeof(int32_t), TexCoordIndex_s15_up); 
  s15->addChild(stripSet);


  // left
  texture_s15 = new SoTexture2;
  s15 -> addChild(texture_s15);
  texture_s15 -> filename.setValue("Texture/s15/texture_s15_left.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s15_indices_left)/sizeof(int32_t), s15_indices_left);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s15_left)/sizeof(int), normalIndex_s15_left);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s15_left)/sizeof(int32_t), TexCoordIndex_s15_left); 
  s15->addChild(stripSet);


  // right
  texture_s15 = new SoTexture2;
  s15 -> addChild(texture_s15);
  texture_s15 -> filename.setValue("Texture/s15/texture_s15_right.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s15_indices_right)/sizeof(int32_t), s15_indices_right);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s15_right)/sizeof(int), normalIndex_s15_right);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s15_right)/sizeof(int32_t), TexCoordIndex_s15_right); 
  s15->addChild(stripSet);

  
  // back
  texture_s15 = new SoTexture2;
  s15 -> addChild(texture_s15);
  texture_s15 -> filename.setValue("Texture/s15/texture_s15_back.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s15_indices_back)/sizeof(int32_t), s15_indices_back);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s15_back)/sizeof(int), normalIndex_s15_back);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s15_back)/sizeof(int32_t), TexCoordIndex_s15_back); 
  s15->addChild(stripSet);


  // bottom
  texture_s15 = new SoTexture2;
  s15 -> addChild(texture_s15);
  texture_s15 -> filename.setValue("Texture/s15/texture_bottom.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s15_indices_bottom)/sizeof(int32_t), s15_indices_bottom);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s15_bottom)/sizeof(int), normalIndex_s15_bottom);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s15_bottom)/sizeof(int32_t), TexCoordIndex_s15_bottom); 
  s15->addChild(stripSet);
 


	s15 -> unrefNoDelete();
	return s15;
}
