#include"buildings.h"

static float s16_vertices[][3] = {
	-1.5f, 0, -0.5f,  // 0
	1.5f, 0, -0.5f,  // 1
    1.5f, 1, -0.5f,  // 2
    -1.5f, 1, -0.5f,  // 3

    -1.5f, 1.5, 0.5f,  // 4
	1.5f, 1.5, 0.5f,  // 5

	1.5f, 1, 0.5f,  // 6
    -1.5f, 1, 0.5f,  // 7
    -1.5f, 0, 0.5f,  // 8
    1.5f, 0, 0.5f,  // 9


};

static float s16_normal[][3] = {

	1, 0, 0, // 0 x 
    0, 1, 0, // 1 y
    0, 0, 1, // 2 z
 
	-1, 0, 0, // 3 -x
    0, -1, 0, // 4 -y
    0, 0, -1, // 5 -z

    0,0.89442f,-0.44721f  // 6 front_up sklon strechy
};

static int32_t s16_indices_front[] = { 1,0,2,3, -1 };
static int32_t s16_indices_up[] = { 2,3,5,4, -1 };
static int32_t s16_indices_left[] = { 5,2,6,1,9, -1 };
static int32_t s16_indices_right[] = { 4,3,7,0,8, -1 };
static int32_t s16_indices_back[] = { 4,5,8,9, -1 };
static int32_t s16_indices_bottom[] = { 0,1,8,9, -1 };

static int normalIndex_s16_front[] = { 5, 5 };
static int normalIndex_s16_up[] = { 6, 6 };
static int normalIndex_s16_left[] = { 0, 0, 0 };
static int normalIndex_s16_right[] = { 3, 3, 3 };
static int normalIndex_s16_back[] = { 2, 2 };
static int normalIndex_s16_bottom[] = { 4, 4 };

static int32_t TexCoordIndex_s16_front[] = { 0,1,2,3, -1 };
static int32_t TexCoordIndex_s16_up[] = { 0,1,2,3, -1 };
static int32_t TexCoordIndex_s16_left[] = { 3,4,5,0,1, -1 };
static int32_t TexCoordIndex_s16_right[] = { 3,4,5,0,1, -1 };
static int32_t TexCoordIndex_s16_back[] = { 2,3,0,1, -1 };
static int32_t TexCoordIndex_s16_bottom[] = { 0,1,2,3, -1 };


SoSeparator *make_s16_FaceSet(float x, float y, float z, float alfa)
{
	SoSeparator *s16 = new SoSeparator();
	s16 -> ref();

    SoTransform *transform = new SoTransform;
    transform->translation.setValue(x,y,z);
    s16->addChild(transform);

    SoScale *scale = new SoScale;
    scale->scaleFactor.setValue(SCALE_X,SCALE_Y,SCALE_Z);
    s16->addChild(scale);

  // rotace modelu budovy
  SoRotationXYZ *myRotation = new SoRotationXYZ;     
  myRotation -> angle = alfa;
  myRotation -> axis = SoRotationXYZ::Y;
  s16 -> addChild(myRotation);

  SoNormal *myNormals = new SoNormal;
  myNormals -> vector.setValues(0, 11, s16_normal);
  s16 -> addChild(myNormals); 

  SoNormalBinding *myNormalBinding = new SoNormalBinding;
  myNormalBinding -> value = SoNormalBinding::PER_FACE_INDEXED;
  s16 -> addChild(myNormalBinding);
  
  SoCoordinate3 *coords = new SoCoordinate3;
  coords->point.setValues(0, sizeof(s16_vertices)/sizeof(float), s16_vertices);
  s16->addChild(coords);

  SoTextureCoordinate2 *texCoord = new SoTextureCoordinate2;
  texCoord->point.set1Value(0, SbVec2f(0,0));
  texCoord->point.set1Value(1, SbVec2f(1,0));
  texCoord->point.set1Value(2, SbVec2f(0,1));
  texCoord->point.set1Value(3, SbVec2f(1,1));
  texCoord->point.set1Value(4, SbVec2f(0,0.6666f));
  texCoord->point.set1Value(5, SbVec2f(1,0.6666f));

  s16->addChild(texCoord);

  // front
  SoTexture2 *texture_s16 = new SoTexture2;
  s16 -> addChild(texture_s16);
  texture_s16 -> filename.setValue("Texture/s16/texture_s16_front.PNG");

  SoIndexedTriangleStripSet *stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s16_indices_front)/sizeof(int32_t), s16_indices_front);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s16_front)/sizeof(int), normalIndex_s16_front);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s16_front)/sizeof(int32_t), TexCoordIndex_s16_front); 
  s16->addChild(stripSet);


  // up
  texture_s16 = new SoTexture2;
  s16 -> addChild(texture_s16);
  texture_s16 -> filename.setValue("Texture/s16/texture_s16_up.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s16_indices_up)/sizeof(int32_t), s16_indices_up);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s16_up)/sizeof(int), normalIndex_s16_up);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s16_up)/sizeof(int32_t), TexCoordIndex_s16_up); 
  s16->addChild(stripSet);


  // left
  texture_s16 = new SoTexture2;
  s16 -> addChild(texture_s16);
  texture_s16 -> filename.setValue("Texture/s16/texture_s16_left.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s16_indices_left)/sizeof(int32_t), s16_indices_left);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s16_left)/sizeof(int), normalIndex_s16_left);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s16_left)/sizeof(int32_t), TexCoordIndex_s16_left); 
  s16->addChild(stripSet);


  // right
  texture_s16 = new SoTexture2;
  s16 -> addChild(texture_s16);
  texture_s16 -> filename.setValue("Texture/s16/texture_s16_right.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s16_indices_right)/sizeof(int32_t), s16_indices_right);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s16_right)/sizeof(int), normalIndex_s16_right);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s16_right)/sizeof(int32_t), TexCoordIndex_s16_right); 
  s16->addChild(stripSet);

  
  // back
  texture_s16 = new SoTexture2;
  s16 -> addChild(texture_s16);
  texture_s16 -> filename.setValue("Texture/s16/texture_s16_back.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s16_indices_back)/sizeof(int32_t), s16_indices_back);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s16_back)/sizeof(int), normalIndex_s16_back);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s16_back)/sizeof(int32_t), TexCoordIndex_s16_back); 
  s16->addChild(stripSet);


  // bottom
  texture_s16 = new SoTexture2;
  s16 -> addChild(texture_s16);
  texture_s16 -> filename.setValue("Texture/s16/texture_bottom.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s16_indices_bottom)/sizeof(int32_t), s16_indices_bottom);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s16_bottom)/sizeof(int), normalIndex_s16_bottom);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s16_bottom)/sizeof(int32_t), TexCoordIndex_s16_bottom); 
  s16->addChild(stripSet);
 


	s16 -> unrefNoDelete();
	return s16;
}
