#include"buildings.h"

static float s17_vertices[][3] = {
	-2, 0, -0.5f,  // 0
	2, 0, -0.5f,  // 1
    2, 1, -0.5f,  // 2
    -2, 1, -0.5f,  // 3

    -2, 1.5, 0.5f,  // 4
	2, 1.5, 0.5f,  // 5

	2, 1, 0.5f,  // 6
    -2, 1, 0.5f,  // 7
    -2, 0, 0.5f,  // 8
    2, 0, 0.5f,  // 9


};

static float s17_normal[][3] = {

	1, 0, 0, // 0 x 
    0, 1, 0, // 1 y
    0, 0, 1, // 2 z
 
	-1, 0, 0, // 3 -x
    0, -1, 0, // 4 -y
    0, 0, -1, // 5 -z

    0,0.89442f,-0.44721f  // 6 front_up sklon strechy
};

static int32_t s17_indices_front[] = { 1,0,2,3, -1 };
static int32_t s17_indices_up[] = { 2,3,5,4, -1 };
static int32_t s17_indices_left[] = { 5,2,6,1,9, -1 };
static int32_t s17_indices_right[] = { 4,3,7,0,8, -1 };
static int32_t s17_indices_back[] = { 4,5,8,9, -1 };
static int32_t s17_indices_bottom[] = { 0,1,8,9, -1 };

static int normalIndex_s17_front[] = { 5, 5 };
static int normalIndex_s17_up[] = { 6, 6 };
static int normalIndex_s17_left[] = { 0, 0, 0 };
static int normalIndex_s17_right[] = { 3, 3, 3 };
static int normalIndex_s17_back[] = { 2, 2 };
static int normalIndex_s17_bottom[] = { 4, 4 };

static int32_t TexCoordIndex_s17_front[] = { 0,1,2,3, -1 };
static int32_t TexCoordIndex_s17_up[] = { 0,1,2,3, -1 };
static int32_t TexCoordIndex_s17_left[] = { 3,4,5,0,1, -1 };
static int32_t TexCoordIndex_s17_right[] = { 3,4,5,0,1, -1 };
static int32_t TexCoordIndex_s17_back[] = { 2,3,0,1, -1 };
static int32_t TexCoordIndex_s17_bottom[] = { 0,1,2,3, -1 };


SoSeparator *make_s17_FaceSet(float x, float y, float z, float alfa)
{
	SoSeparator *s17 = new SoSeparator();
	s17 -> ref();

    SoTransform *transform = new SoTransform;
    transform->translation.setValue(x,y,z);
    s17->addChild(transform);

    SoScale *scale = new SoScale;
    scale->scaleFactor.setValue(SCALE_X,SCALE_Y,SCALE_Z);
    s17->addChild(scale);

  // rotace modelu budovy
  SoRotationXYZ *myRotation = new SoRotationXYZ;     
  myRotation -> angle = alfa;
  myRotation -> axis = SoRotationXYZ::Y;
  s17 -> addChild(myRotation);

  SoNormal *myNormals = new SoNormal;
  myNormals -> vector.setValues(0, 11, s17_normal);
  s17 -> addChild(myNormals); 

  SoNormalBinding *myNormalBinding = new SoNormalBinding;
  myNormalBinding -> value = SoNormalBinding::PER_FACE_INDEXED;
  s17 -> addChild(myNormalBinding);
  
  SoCoordinate3 *coords = new SoCoordinate3;
  coords->point.setValues(0, sizeof(s17_vertices)/sizeof(float), s17_vertices);
  s17->addChild(coords);

  SoTextureCoordinate2 *texCoord = new SoTextureCoordinate2;
  texCoord->point.set1Value(0, SbVec2f(0,0));
  texCoord->point.set1Value(1, SbVec2f(1,0));
  texCoord->point.set1Value(2, SbVec2f(0,1));
  texCoord->point.set1Value(3, SbVec2f(1,1));
  texCoord->point.set1Value(4, SbVec2f(0,0.6666f));
  texCoord->point.set1Value(5, SbVec2f(1,0.6666f));

  s17->addChild(texCoord);

  // front
  SoTexture2 *texture_s17 = new SoTexture2;
  s17 -> addChild(texture_s17);
  texture_s17 -> filename.setValue("Texture/s17/texture_s17_front.PNG");

  SoIndexedTriangleStripSet *stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s17_indices_front)/sizeof(int32_t), s17_indices_front);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s17_front)/sizeof(int), normalIndex_s17_front);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s17_front)/sizeof(int32_t), TexCoordIndex_s17_front); 
  s17->addChild(stripSet);


  // up
  texture_s17 = new SoTexture2;
  s17 -> addChild(texture_s17);
  texture_s17 -> filename.setValue("Texture/s17/texture_s17_up.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s17_indices_up)/sizeof(int32_t), s17_indices_up);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s17_up)/sizeof(int), normalIndex_s17_up);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s17_up)/sizeof(int32_t), TexCoordIndex_s17_up); 
  s17->addChild(stripSet);


  // left
  texture_s17 = new SoTexture2;
  s17 -> addChild(texture_s17);
  texture_s17 -> filename.setValue("Texture/s17/texture_s17_left.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s17_indices_left)/sizeof(int32_t), s17_indices_left);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s17_left)/sizeof(int), normalIndex_s17_left);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s17_left)/sizeof(int32_t), TexCoordIndex_s17_left); 
  s17->addChild(stripSet);


  // right
  texture_s17 = new SoTexture2;
  s17 -> addChild(texture_s17);
  texture_s17 -> filename.setValue("Texture/s17/texture_s17_right.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s17_indices_right)/sizeof(int32_t), s17_indices_right);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s17_right)/sizeof(int), normalIndex_s17_right);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s17_right)/sizeof(int32_t), TexCoordIndex_s17_right); 
  s17->addChild(stripSet);

  
  // back
  texture_s17 = new SoTexture2;
  s17 -> addChild(texture_s17);
  texture_s17 -> filename.setValue("Texture/s17/texture_s17_back.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s17_indices_back)/sizeof(int32_t), s17_indices_back);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s17_back)/sizeof(int), normalIndex_s17_back);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s17_back)/sizeof(int32_t), TexCoordIndex_s17_back); 
  s17->addChild(stripSet);


  // bottom
  texture_s17 = new SoTexture2;
  s17 -> addChild(texture_s17);
  texture_s17 -> filename.setValue("Texture/s17/texture_bottom.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s17_indices_bottom)/sizeof(int32_t), s17_indices_bottom);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s17_bottom)/sizeof(int), normalIndex_s17_bottom);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s17_bottom)/sizeof(int32_t), TexCoordIndex_s17_bottom); 
  s17->addChild(stripSet);
 


	s17 -> unrefNoDelete();
	return s17;
}
