#include"buildings.h"

static float s18_vertices[][3] = {
	-1, 0, -0.5f,  // 0
	1, 0, -0.5f,  // 1
    1, 2, -0.5f,  // 2
    -1, 2, -0.5f,  // 3

    -1, 2.5, 0.5f,  // 4
	1, 2.5, 0.5f,  // 5

	1, 2, 0.5f,  // 6
    -1, 2, 0.5f,  // 7
    -1, 0, 0.5f,  // 8
    1, 0, 0.5f,  // 9


};

static float s18_normal[][3] = {

	1, 0, 0, // 0 x 
    0, 1, 0, // 1 y
    0, 0, 1, // 2 z
 
	-1, 0, 0, // 3 -x
    0, -1, 0, // 4 -y
    0, 0, -1, // 5 -z

    0,0.89442f,-0.44721f  // 6 front_up sklon strechy
};

static int32_t s18_indices_front[] = { 1,0,2,3, -1 };
static int32_t s18_indices_up[] = { 2,3,5,4, -1 };
static int32_t s18_indices_left[] = { 5,2,6,1,9, -1 };
static int32_t s18_indices_right[] = { 4,3,7,0,8, -1 };
static int32_t s18_indices_back[] = { 4,5,8,9, -1 };
static int32_t s18_indices_bottom[] = { 0,1,8,9, -1 };

static int normalIndex_s18_front[] = { 5, 5 };
static int normalIndex_s18_up[] = { 6, 6 };
static int normalIndex_s18_left[] = { 0, 0, 0 };
static int normalIndex_s18_right[] = { 3, 3, 3 };
static int normalIndex_s18_back[] = { 2, 2 };
static int normalIndex_s18_bottom[] = { 4, 4 };

static int32_t TexCoordIndex_s18_front[] = { 0,1,2,3, -1 };
static int32_t TexCoordIndex_s18_up[] = { 0,1,2,3, -1 };
static int32_t TexCoordIndex_s18_left[] = { 3,4,5,0,1, -1 };
static int32_t TexCoordIndex_s18_right[] = { 3,4,5,0,1, -1 };
static int32_t TexCoordIndex_s18_back[] = { 2,3,0,1, -1 };
static int32_t TexCoordIndex_s18_bottom[] = { 0,1,2,3, -1 };


SoSeparator *make_s18_FaceSet(float x, float y, float z, float alfa)
{
	SoSeparator *s18 = new SoSeparator();
	s18 -> ref();

    SoTransform *transform = new SoTransform;
    transform->translation.setValue(x,y,z);
    s18->addChild(transform);

    SoScale *scale = new SoScale;
    scale->scaleFactor.setValue(SCALE_X,SCALE_Y,SCALE_Z);
    s18->addChild(scale);

  // rotace modelu budovy
  SoRotationXYZ *myRotation = new SoRotationXYZ;     
  myRotation -> angle = alfa;
  myRotation -> axis = SoRotationXYZ::Y;
  s18 -> addChild(myRotation);

  SoNormal *myNormals = new SoNormal;
  myNormals -> vector.setValues(0, 11, s18_normal);
  s18 -> addChild(myNormals); 

  SoNormalBinding *myNormalBinding = new SoNormalBinding;
  myNormalBinding -> value = SoNormalBinding::PER_FACE_INDEXED;
  s18 -> addChild(myNormalBinding);
  
  SoCoordinate3 *coords = new SoCoordinate3;
  coords->point.setValues(0, sizeof(s18_vertices)/sizeof(float), s18_vertices);
  s18->addChild(coords);

  SoTextureCoordinate2 *texCoord = new SoTextureCoordinate2;
  texCoord->point.set1Value(0, SbVec2f(0,0));
  texCoord->point.set1Value(1, SbVec2f(1,0));
  texCoord->point.set1Value(2, SbVec2f(0,1));
  texCoord->point.set1Value(3, SbVec2f(1,1));
  texCoord->point.set1Value(4, SbVec2f(0,0.6666f));
  texCoord->point.set1Value(5, SbVec2f(1,0.6666f));

  s18->addChild(texCoord);

  // front
  SoTexture2 *texture_s18 = new SoTexture2;
  s18 -> addChild(texture_s18);
  texture_s18 -> filename.setValue("Texture/s18/texture_s18_front.PNG");

  SoIndexedTriangleStripSet *stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s18_indices_front)/sizeof(int32_t), s18_indices_front);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s18_front)/sizeof(int), normalIndex_s18_front);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s18_front)/sizeof(int32_t), TexCoordIndex_s18_front); 
  s18->addChild(stripSet);


  // up
  texture_s18 = new SoTexture2;
  s18 -> addChild(texture_s18);
  texture_s18 -> filename.setValue("Texture/s18/texture_s18_up.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s18_indices_up)/sizeof(int32_t), s18_indices_up);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s18_up)/sizeof(int), normalIndex_s18_up);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s18_up)/sizeof(int32_t), TexCoordIndex_s18_up); 
  s18->addChild(stripSet);


  // left
  texture_s18 = new SoTexture2;
  s18 -> addChild(texture_s18);
  texture_s18 -> filename.setValue("Texture/s18/texture_s18_left.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s18_indices_left)/sizeof(int32_t), s18_indices_left);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s18_left)/sizeof(int), normalIndex_s18_left);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s18_left)/sizeof(int32_t), TexCoordIndex_s18_left); 
  s18->addChild(stripSet);


  // right
  texture_s18 = new SoTexture2;
  s18 -> addChild(texture_s18);
  texture_s18 -> filename.setValue("Texture/s18/texture_s18_right.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s18_indices_right)/sizeof(int32_t), s18_indices_right);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s18_right)/sizeof(int), normalIndex_s18_right);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s18_right)/sizeof(int32_t), TexCoordIndex_s18_right); 
  s18->addChild(stripSet);

  
  // back
  texture_s18 = new SoTexture2;
  s18 -> addChild(texture_s18);
  texture_s18 -> filename.setValue("Texture/s18/texture_s18_back.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s18_indices_back)/sizeof(int32_t), s18_indices_back);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s18_back)/sizeof(int), normalIndex_s18_back);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s18_back)/sizeof(int32_t), TexCoordIndex_s18_back); 
  s18->addChild(stripSet);


  // bottom
  texture_s18 = new SoTexture2;
  s18 -> addChild(texture_s18);
  texture_s18 -> filename.setValue("Texture/s18/texture_bottom.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s18_indices_bottom)/sizeof(int32_t), s18_indices_bottom);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s18_bottom)/sizeof(int), normalIndex_s18_bottom);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s18_bottom)/sizeof(int32_t), TexCoordIndex_s18_bottom); 
  s18->addChild(stripSet);
 


	s18 -> unrefNoDelete();
	return s18;
}
